import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

class FirmaBin {

  final static char FELT_SLUTT_TEGN    = '|';

  // Felt
  private PersonellRegister register;

  // Konstruktrer
  FirmaBin() {
    register = new PersonellRegister();
  }
  FirmaBin(PersonellRegister register) {
    this.register = register;
  }

  // Skriv statistikk
  void skrivRapport() {
    System.out.println(register);
  }

  void skrivAlleAnsatteTilTerminalvindu() {
    Ansatt[] ansattTabell = register.hentAnsattTabell();
    int antallAnsatte = register.hentAntallAnsatte();
    System.out.println("Antall ansatte: " + antallAnsatte);
    for (int i = 0; i < antallAnsatte; i++)
      System.out.println(ansattTabell[i]);
  }

  public static void main(String[] args) {
    // Opprett en tabell med ansatte.
    Ansatt[] ansattInfo = {
        new Ansatt("Ole",  "Olsen",  515.00, Ansatt.MANN),
        new Ansatt("Bill", "Bailey", 612.00, Ansatt.MANN),
        new Ansatt("Liv",  "Larsen", 820.00, Ansatt.KVINNE)
    };

    // Opprett et personellregister.
    PersonellRegister register = new PersonellRegister(ansattInfo);
    // Opprett et firma.
    FirmaBin firma = new FirmaBin(register);

    // Skriv ansattopplysninger i personellregisteret til en fil.           (1)
    firma.skrivAlleAnsatteTilBinFil("ansattfil.dat", false);

    // Les ansattopplysninger fra en fil til et nytt personellregister
    System.out.println("Leser fra binrfil.");
    firma.lesAlleAnsatteFraBinFil("ansattfil.dat");                      // (2)

    // Skriv opplysninger om ansatte til terminalvinduet.
    firma.skrivAlleAnsatteTilTerminalvindu();
    firma.skrivRapport();
  }

  void skrivAlleAnsatteTilBinFil(String datafilnavn, boolean utvid) {
    try (
      FileOutputStream utFilStrm = new FileOutputStream(datafilnavn, utvid);    // (3)
      BufferedOutputStream bufretUtStrm = new BufferedOutputStream(utFilStrm); // (3a)
      DataOutputStream utBinStrm = new DataOutputStream(bufretUtStrm) ) {      // (4)
      // Skriv hvor mange poster som er i filen.                         // (5)
      int antallAnsatte = register.hentAntallAnsatte();
      utBinStrm.writeInt(antallAnsatte);

      // Skriv ut informasjon om hver ansatt.
      Ansatt[] ansattTabell = register.hentAnsattTabell();
      for (int i = 0; i < antallAnsatte; i++)
        skrivAnsattData(utBinStrm, ansattTabell[i]);                    // (6)
    }
    catch (FileNotFoundException unntak) {                               // (8)
      System.out.print("Feil ved pning av fil: " + datafilnavn);
    }
    catch (IOException unntak) {                                         // (9)
      System.out.print("Feil ved skriving: " + unntak);
    }
  }

  void skrivAnsattData(DataOutputStream utBinStrm, Ansatt ansatt)       // (10)
       throws IOException {
    //  Et strengfelt avsluttes med FELT_SLUTT_TEGN.
    utBinStrm.writeChars(ansatt.hentFornavn()    + FELT_SLUTT_TEGN);
    utBinStrm.writeChars(ansatt.hentEtternavn()  + FELT_SLUTT_TEGN);
    utBinStrm.writeDouble(ansatt.hentTimelnn());
    utBinStrm.writeBoolean(ansatt.hentKjnn());
  }

  void lesAlleAnsatteFraBinFil(String datafilnavn) {
    try (
      FileInputStream innFilStrm = new FileInputStream(datafilnavn);            // (11)
      BufferedInputStream bufretInnStrm = new BufferedInputStream(innFilStrm); // (11a)
      DataInputStream innBinStrm = new DataInputStream(bufretInnStrm) ) {      // (12)

      // Les hvor mange poster som er i filen.
      int totalAnsatte = innBinStrm.readInt();                          // (13)

      // Opprett et nytt personellregister
      register = new PersonellRegister();

      // Les inn informasjon om hver ansatt.
      for (int i = 0; i < totalAnsatte; i++) {
        // Les data i den.
        Ansatt nyAnsatt = lesAnsattData(innBinStrm);                    // (14)
        // Innsett i registeret.
        register.leggTilAnsatt(nyAnsatt);
      }
    }
    catch (IOException unntak) {
      System.out.print("Feil ved lesing: " + unntak);
    }
  }

  Ansatt lesAnsattData(DataInputStream innBinStrm) throws IOException { // (16)
    String fornavn   = lesBinStrengFelt(innBinStrm);
    String etternavn = lesBinStrengFelt(innBinStrm);
    double timelnn  = innBinStrm.readDouble();
    boolean kjnn    = innBinStrm.readBoolean();
    return new Ansatt(fornavn, etternavn, timelnn, kjnn);
  }

  private String lesBinStrengFelt(DataInputStream innBinStrm)           // (17)
                 throws IOException {
    String str = "";
    while(true) {
      char tegn = innBinStrm.readChar();
      if (tegn == FELT_SLUTT_TEGN)
        break;
      str += tegn;
    }
    return str;
  }
}