import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

class FirmaDirekte {

  final static char FELT_SLUTT_TEGN    = '|';

  // Konstanter
  final static boolean MANN   = false;
  final static boolean KVINNE = true;
  final static double  NORMAL_ARBEIDSUKE = 37.5;

  // Konstanter for hndtering av poster med fast-lengde.                   (1)
  final static char FYLLE_TEGN         = '\u0000';
  final static int  FORNAVN_LENGDE     = 20; // Unicode-tegn
  final static int  ETTERNAVN_LENGDE   = 30; // Unicode-tegn
  final static int  TIMELNN_LENGDE    = 8;  // bytes
  final static int  KJNN_LENGDE       = 1;  // byte
  final static int  POST_LENGDE_BYTES  = 2*FORNAVN_LENGDE +
                                         2*ETTERNAVN_LENGDE +
                                           TIMELNN_LENGDE +
                                           KJNN_LENGDE; // bytes
  // Direkte tilgang-fil.
  private RandomAccessFile raf;

  // Register med ansatte
  private PersonellRegister register;

  // Konstruktrer
  FirmaDirekte() {
    register = new PersonellRegister();
  }
  FirmaDirekte(PersonellRegister register) {
    this.register = register;
  }

  // Metoder som skriver statistikk
  void skrivRapport() {
    System.out.println(register);
  }
  void skrivAlleAnsatteTilTerminalvindu() {
    Ansatt[] ansattTabell = register.hentAnsattTabell();
    int antallAnsatte = register.hentAntallAnsatte();
    System.out.println("Antall ansatte: " + antallAnsatte);
    for (int i = 0; i < antallAnsatte; i++)
      System.out.println(ansattTabell[i]);
  }

  public static void main(String[] args) {
    // Opprett en tabell med ansatte.
    Ansatt[] ansattInfo = {
        new Ansatt("Ole", "Olsen", 515.00, Ansatt.MANN),
        new Ansatt("Bill", "Bailey", 612.00, Ansatt.MANN),
        new Ansatt("Liv", "Larsen", 820.00, Ansatt.KVINNE)
    };

    // Opprett et personellregister.
    PersonellRegister register = new PersonellRegister(ansattInfo);
    // Opprett et firma.
    FirmaDirekte firma = new FirmaDirekte(register);                     // (2)
    System.out.println("Ansatte som skal lagres p fil:");
    firma.skrivAlleAnsatteTilTerminalvindu();

    // Tilrettelegg for lesing og skriving.
    firma.initDirekteTilgangFil("ansattfil.data");                       // (3)

    // Skriv alle ansatte til filen.
    int antallAnsatte = register.hentAntallAnsatte();
    for (int i = 0; i < antallAnsatte; i++)
      firma.utvidFilMedEnAnsatt(register.hentAnsatt(i));                 // (4)

    // Oppdater timelnn direkte p fil.
    firma.registrerNyTimeLnnForAnsattPFil(2, 100.0);                   // (5)
    firma.registrerNyTimeLnnForAnsattPFil(0, 300.0);
    firma.registrerNyTimeLnnForAnsattPFil(1, 400.0);

    System.out.println("Leser alle ansatte fra direkte tilgang-fil.");
    firma.lesAlleAnsatteFraDirekteTilgangFil();                          // (6)

    // Skriv statistikk
    firma.skrivAlleAnsatteTilTerminalvindu();
    firma.skrivRapport();

    firma.lukkStrm();
  }

  /** Metoden pner filen for lesing og skriving. */
  void initDirekteTilgangFil(String datafilnavn) {                       // (7)

    try {
      // Opprett fil for lesing og skriving.
      raf = new RandomAccessFile(datafilnavn, "rw");
      // Nullstill filen.
      raf.setLength(0);
      // Oppdater informasjon om antall poster i filen.
      raf.writeInt(0);
    }
    catch (FileNotFoundException unntak) {
      System.out.print("Filen ikke funnet." + unntak);
      System.exit(1);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved skriving: " + unntak);
      System.exit(1);
    }
  }

  /** Metoden utvider filen med en ansatt-post. */
  void utvidFilMedEnAnsatt(Ansatt ansatt) {                              // (8)
    try {
      // Posisjoner filpeker etter siste post.
      raf.seek(raf.length());
      // Skriv ansatt-info til filen.
      skrivAnsattData(ansatt);

      // Oppdater antall poster lagret p filen.
      raf.seek(0);
      int antallRegistrert = raf.readInt();
      raf.seek(0);
      raf.writeInt(antallRegistrert + 1);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved skriving: " + unntak);
    }
  }

  /** Metoden skriver feltverdier til en ansatt. */
  void skrivAnsattData(Ansatt ansatt) throws IOException {
    skrivFastLengdeStreng(ansatt.hentFornavn(), FORNAVN_LENGDE);
    skrivFastLengdeStreng(ansatt.hentEtternavn(), ETTERNAVN_LENGDE);
    raf.writeDouble(ansatt.hentTimelnn());
    raf.writeBoolean(ansatt.hentKjnn());
  }

  /** Metoden skriver en streng som opptar maks feltlengde. */
  private void skrivFastLengdeStreng(String str, int maksFeltLengde)     // (9)
                                     throws IOException {
    //  Feltet blir utvidet med FYLLE_TEGN dersom lengden
    //  er mindre enn MAKS_FELT_LENGDE.
    int antallFylleTegn = maksFeltLengde - str.length();
    for (int k = 0; k < antallFylleTegn; k++)
      str += FYLLE_TEGN;
    raf.writeChars(str.substring(0, maksFeltLengde));
  }

  /** Metoden utvider filen med en ansatt-post. */
  void registrerNyTimeLnnForAnsattPFil(int ansattNummer, double tLnn) {// (10)
    try {
      // Sett filpeker til riktig post.
      raf.seek( 4 + ansattNummer * POST_LENGDE_BYTES);
      // Les post i et "tomt" ansatt-objekt og oppdater lnn.
      Ansatt enAnsatt = lesAnsattData();                                 // (11)
      enAnsatt.settTimelnn(tLnn);

      // Posisjoner filpekeren til  skrive tilbake posten.
      long innevrendePosisjon = raf.getFilePointer();
      raf.seek(innevrendePosisjon - POST_LENGDE_BYTES);
      skrivAnsattData(enAnsatt);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved innlesing: " + unntak);
    }
  }

  /** Metoden leser en ansatt-post fra innevrende posisjon i filen. */
  Ansatt lesAnsattData() throws IOException {
    String fornavn   = lesFastLengdeStreng(FORNAVN_LENGDE);
    String etternavn = lesFastLengdeStreng(ETTERNAVN_LENGDE);
    double timelnn  = raf.readDouble();
    boolean kjnn    = raf.readBoolean();
    return new Ansatt(fornavn, etternavn, timelnn, kjnn);
  }

  /** Metoden leser en streng som opptar maks feltlengde. */
  private String lesFastLengdeStreng(int maksFeltLengde) throws IOException {// (12)
    //  Les tegn som utgjr strengverdien.
    String str = "";
    int i;
    for (i = 0; i < maksFeltLengde; i++) {
      char tegn = raf.readChar();
      if (tegn == FYLLE_TEGN)
        break;
      else
        str += tegn;
    }
    //  Hopp over ev. sekvens av FYLLE_TEGN som opptar 2 bytes hver.
    raf.skipBytes(2*(maksFeltLengde - i - 1));
    return str;
  }

  /** Metoden leser alle ansatt-poster. */
  void lesAlleAnsatteFraDirekteTilgangFil() {                            // (13)
    try {
      // Sett filpekeren til starten.
      raf.seek(0);

      // Opprett nytt personell-register
      register = new PersonellRegister();

      // Les hvor mange poster som er i filen.
      int totalAnsatte = raf.readInt();

      // Les inn informasjon om hver ansatt sekvensielt.
      for (int i = 0; i < totalAnsatte; i++) {
        // Les data on en ansatt.
        Ansatt nyAnsatt = lesAnsattData();
        // Innsett i registeret.
        register.leggTilAnsatt(nyAnsatt);
      }
    }
    catch (IOException unntak) {
      System.out.print("Feil ved lesing: " + unntak);
      System.exit(1); // avbryter dersom IO-unntak oppstr.
    }
  }

  /** Metoden lukker filen for direkte tilgang. */
  void lukkStrm() {
    try {
      if (raf != null)
        raf.close();
    }
    catch (IOException unntak) {
      System.out.print("Feil ved lukking: " + unntak);
      System.exit(1);
    }
  }

  /** Metoden endrer lnn til en ansatt angitt med ansattnummer. */
  void settTimeLnn(int ansattNummer, double tLnn) {
    try {
      register.hentAnsatt(ansattNummer).settTimelnn(tLnn);
      skrivInfoOmAnsattNummer(ansattNummer);
    }
    catch (ArrayIndexOutOfBoundsException unntak) {
      System.out.print("Ulovlig ansatt-nummer: " + unntak);
    }
  }

  /** Metoden skriver opplysninger om en ansatt angitt med ansattnummer. */
  void skrivInfoOmAnsattNummer(int ansattNummer) {
    try {
      // Posisjoner filpeker.
      // M legge til 4 bytes (int) ved starten av filen som brukes
      // til  lagre hvor manger poster det er i filen.
      raf.seek( 4 + (ansattNummer)* POST_LENGDE_BYTES);
      // Skriv ansatt-info til filen.
      skrivAnsattData(register.hentAnsatt(ansattNummer));
    }
    catch (IOException unntak) {
      System.out.print("Feil ved skriving: " + unntak);
    }
    catch (ArrayIndexOutOfBoundsException unntak) {
      System.out.print("Ulovlig ansatt-nummer: " + unntak);
    }
  }

  /** Metoden skriver opplysninger om alle ansatte. */
  void skrivAlleAnsatteTilDirekteTilgangFil() {
    try {
      // Sett filpekeren til starten.
      raf.seek(0);

      // Skriv hvor mange poster (dvs. antall ansatte) som skal
      // lagres i filen.
      int antallAnsatte = register.hentAntallAnsatte();
      raf.writeInt(antallAnsatte);

      // Skriv ut informasjon om alle ansatte.
      Ansatt[] ansattTabell = register.hentAnsattTabell();
      for (int i = 0; i < antallAnsatte; i++)
        skrivAnsattData(ansattTabell[i]);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved skriving: " + unntak);
      System.exit(1); // avbryter dersom IO-unntak oppstr.
    }
  }

  /** Metoden leser opplysninger om en ansatt angitt med ansattnummer. */
  void lesInfoOmAnsattNummer(int ansattNummer) {
    try {
      // Posisjoner filpeker.
      // M legge til 4 bytes (int) ved starten av filen som brukes
      // til  lagre hvor manger poster det er i filen.
      raf.seek( 4 + (ansattNummer)* POST_LENGDE_BYTES);
      // Les data om en ansatt.
      Ansatt nyAnsatt = lesAnsattData();
      // Innsett i registeret.
      register.settAnsatt(ansattNummer, nyAnsatt);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved innlesing: " + unntak);
    }
  }
}