import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

class FirmaObj {

  // Felt
  private PersonellRegister register;

  // Konstruktrer
  FirmaObj() {
    register = new PersonellRegister();
  }
  FirmaObj(PersonellRegister register) {
    this.register = register;
  }

  // Skriv statistikk
  void skrivRapport() {
    System.out.println(register);
  }
  void skrivAlleAnsatteTilTerminalvindu() {
    Ansatt[] ansattTabell = register.hentAnsattTabell();
    int antallAnsatte = register.hentAntallAnsatte();
    System.out.println("Antall ansatte: " + antallAnsatte);
    for (int i = 0; i < antallAnsatte; i++)
      System.out.println(ansattTabell[i]);
  }

  public static void main(String[] args) {
    // Opprett en tabell med forskjellige typer ansatte.
    Ansatt[] ansattInfo = {
        new Ansatt("Ole", "Olsen", 515.00, Ansatt.MANN),
        new FastAnsatt("Bill", "Bailey", 612.00, Ansatt.MANN),
        new AkkordArbeider("Liv", "Larsen", Ansatt.KVINNE, 25.00, 75),
        new ArbeidsLeder("Molly", "Malone", 950.00, Ansatt.KVINNE, 150.00),
        new TimebetaltArbeider("Sally", "Saggu", 120.00, Ansatt.KVINNE)
    };

    // Opprett et personellregister.
    PersonellRegister register = new PersonellRegister(ansattInfo);
    // Opprett et firma.
    FirmaObj firma = new FirmaObj(register);

    // Skriv ansattopplysninger i personellregisteret til en fil.
    firma.skrivAlleAnsatteTilObjFil("ansattfil.obj", false);             // (1)

    // Les ansattopplysninger fra en fil til et nytt personellregister.
    System.out.println("Leser fra objektfil.");
    firma.lesAlleAnsatteFraObjFil("ansattfil.obj");                      // (2)

    // Skriv opplysninger om ansatte til terminalvinduet.
    firma.skrivAlleAnsatteTilTerminalvindu();
    firma.skrivRapport();
  }

  /**
   * Metoden serialiserer ansattobjekter.
   */
  void skrivAlleAnsatteTilObjFil(String datafilnavn, boolean utvid) {
    try (
      FileOutputStream utFilStrm = new FileOutputStream(datafilnavn, utvid);    // (3)
      BufferedOutputStream bufretUtStrm = new BufferedOutputStream(utFilStrm); // (3a)
      ObjectOutputStream utObjStrm = new ObjectOutputStream(bufretUtStrm) ) {                  // (4)

      Ansatt[] ansattTabell = register.hentAnsattTabell();
      int antallAnsatte = register.hentAntallAnsatte();
      int antallKvinner = register.hentAntallKvinner();
      utObjStrm.writeObject(ansattTabell);                             // (5)
      utObjStrm.writeInt(antallAnsatte);                               // (6)
      utObjStrm.writeInt(antallKvinner);                               // (7)
    }
    catch (FileNotFoundException unntak) {
      System.out.print("Kan ikke pne fil: " + datafilnavn);
      System.exit(1);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved skriving: " + unntak);
      System.exit(1);
    }
  }

  /**
   * Metoden deserialiserer ansattobjekter.
   */
  void lesAlleAnsatteFraObjFil(String datafilnavn) {
    try (
      FileInputStream innFilStrm = new FileInputStream(datafilnavn);            // (9)
      BufferedInputStream bufretInnStrm = new BufferedInputStream(innFilStrm); // (10)
      ObjectInputStream innObjStrm = new ObjectInputStream(bufretInnStrm) ) {                 // (10)

      Ansatt[] ansattTabell = null;
      Object nyttObj = innObjStrm.readObject();                        // (11)
      if (nyttObj instanceof Ansatt[])                                  // (12)
        ansattTabell = (Ansatt[]) nyttObj;
      else
        System.out.println("Ansatt-tabell ikke funnet.");
      int antallAnsatte = innObjStrm.readInt();                        // (13)
      int antallKvinner = innObjStrm.readInt();                        // (14)
      register = new PersonellRegister(
                     ansattTabell, antallAnsatte, antallKvinner);       // (15)
    }
    catch (FileNotFoundException unntak) {
      System.out.print("Kan ikke pne fil: " + datafilnavn);
      System.exit(1);
    }
    catch (IOException unntak) {
      System.out.print("Feil ved lesing: " + unntak);
      System.exit(1);
    }
    catch (ClassNotFoundException unntak) {
      System.out.print("Klassedefinisjon ikke funnet: " + unntak);
      System.exit(1);
    }
  }
}
