import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

class FirmaTxtARH {

  final static char FELT_SLUTT_TEGN    = '|';

  // Felt
  private PersonellRegister register;

  // Konstruktrer
  FirmaTxtARH() {
    register = new PersonellRegister();
  }
  FirmaTxtARH(PersonellRegister register) {
    this.register = register;
  }

  // Skriv statistikk
  void skrivRapport() {
    System.out.println(register);
  }
  void skrivAlleAnsatteTilTerminalvindu() {
    Ansatt[] ansattTabell = register.hentAnsattTabell();
    int antallAnsatte = register.hentAntallAnsatte();
    System.out.println("Antall ansatte: " + antallAnsatte);
    for (int i = 0; i < antallAnsatte; i++)
      System.out.println(ansattTabell[i]);
  }

  public static void main(String[] args) {
    // Opprett en tabell med ansatte.
    Ansatt[] ansattInfo = {
        new Ansatt("Ole",  "Olsen",  515.00, Ansatt.MANN),
        new Ansatt("Bill", "Bailey", 612.00, Ansatt.MANN),
        new Ansatt("Liv",  "Larsen", 820.00, Ansatt.KVINNE)
    };

    // Opprett et personellregister.
    PersonellRegister register = new PersonellRegister(ansattInfo);
    // Opprett et firma.
    FirmaTxtARH firma = new FirmaTxtARH(register);

    // Skriv ansattopplysninger i personellregisteret til en fil.
    firma.skrivAlleAnsatteTilTekstFil("ansattfil.txt", false);           // (1)

    // Les ansattopplysninger fra en fil.
    System.out.println("Leser fra tekstfil.");
    firma.lesAlleAnsatteFraTekstFil("ansattfil.txt");                    // (2)

    // Skriv opplysninger om ansatte til terminalvinduet.
    firma.skrivAlleAnsatteTilTerminalvindu();
    firma.skrivRapport();
  }

  void skrivAlleAnsatteTilTekstFil(String datafilnavn, boolean utvid) {
    try ( FileWriter tekstFilSkriver = new FileWriter(datafilnavn,utvid);     // (3)
          BufferedWriter bufretSkriver = new BufferedWriter(tekstFilSkriver);// (3a)
          PrintWriter tekstSkriver = new PrintWriter(bufretSkriver) ){       // (4)

      // Skriv ut hvor mange poster som er i registeret.
      int antallAnsatte = register.hentAntallAnsatte();
      tekstSkriver.println(antallAnsatte);                              // (5)

      // Skriv ut informasjon om hver ansatt.
      Ansatt[] ansattTabell = register.hentAnsattTabell();
      for (int i = 0; i < antallAnsatte; i++)
        skrivAnsattData(tekstSkriver, ansattTabell[i]);                 // (6)
    }
    catch (IOException unntak) {
      System.out.println("Feil ved skriving: " + unntak);
    }
  }

  void skrivAnsattData(PrintWriter tekstSkriver, Ansatt ansatt) {       // (8)
    // Felt adskilt med et feltslutt-tegn.
    tekstSkriver.print(ansatt.hentFornavn()   + FELT_SLUTT_TEGN);
    tekstSkriver.print(ansatt.hentEtternavn() + FELT_SLUTT_TEGN);
    tekstSkriver.print(ansatt.hentTimelnn());
    tekstSkriver.print(FELT_SLUTT_TEGN);
    tekstSkriver.println(ansatt.hentKjnn());
  }

  void lesAlleAnsatteFraTekstFil(String datafilnavn) {
    try ( FileReader tekstFilLeser = new FileReader(datafilnavn);           // (9)
          BufferedReader tekstLeser = new BufferedReader(tekstFilLeser);) { // (10)

      // Les hvor mange poster som er i filen.
      int totalAnsatte = Integer.parseInt(tekstLeser.readLine());       // (11)

      // Opprett et nytt personellregister
      register = new PersonellRegister();

      // Les inn informasjon om alle ansatt p filen.
      for (int i = 0; i < totalAnsatte; i++) {
        // Les en ansatt.
        Ansatt ansatt = lesAnsattData(tekstLeser);                      // (12)
        // Innsett i registeret.
        register.leggTilAnsatt(ansatt);
      }
    }
    catch (IOException unntak) {
      System.out.print("Feil ved lesing: " + unntak);
    }
  }

  Ansatt lesAnsattData(BufferedReader tekstLeser) throws IOException {  // (14)
    // Les en post (som en tekstlinje).
    String post = tekstLeser.readLine();                                // (15)

    // Vi spalter tekstlinjen (dvs en post) vha split()-metoden som returnerer
    // en tabell av strenger som tilsvarer felt i posten.
    String[] felt = post.split("\\" + FELT_SLUTT_TEGN);                 // (16)

    // Les de forskjellige feltene.
    String fornavn   = felt[0];                                         // (17)
    String etternavn = felt[1];
    double timelnn  = Double.parseDouble(felt[2]);                     // (18)
    boolean kjnn = felt[3].equals("true");                             // (19)

    return new Ansatt(fornavn, etternavn, timelnn, kjnn);
  }
}