
class PersonellRegister {

  final static int MAKS_ANTALL_ANSATTE = 100;

  // Feltdeklarasjoner
  private Ansatt[] ansattTabell;
  private int      antallAnsatte;
  private int      antallKvinner;

  // Standardkonstruktr
  PersonellRegister() {
    ansattTabell = new Ansatt[MAKS_ANTALL_ANSATTE];
  }

  // Oppretter personellregister fra en tabell med ansatte
  PersonellRegister(Ansatt[] angittAnsattTabell) {
    ansattTabell = new Ansatt[MAKS_ANTALL_ANSATTE];
    for( int i = 0; i < angittAnsattTabell.length; i++)
      leggTilAnsatt(angittAnsattTabell[i]);
  }

  // Konstruktr som mottar feltverdier som parametere.
  PersonellRegister(Ansatt[] ansattTabell, int antallAnsatte,
      int antallKvinner) {
    this.ansattTabell = ansattTabell;
    this.antallAnsatte = antallAnsatte;
    this.antallKvinner = antallKvinner;
  }

  // Er tabell med ansatte full?
  boolean ansattTabellFull() {
    return antallAnsatte >= ansattTabell.length;
  }

  // Er tabell med ansatte tom?
  boolean ansattTabellTom() {
    return antallAnsatte <= 0;
  }

  // Er lovlig indeks i ansatt tabell?
  boolean lovligIndeks(int indeks) {
    return 0 <= indeks && indeks <= antallAnsatte;
  }

  // Setter inn en ansatt i ansatt-tabellen.
  void leggTilAnsatt(Ansatt nyAnsatt) {
    if (ansattTabellFull()) {
      System.out.println("Ikke plass til flere ansatte.");
      return;
    }
    ansattTabell[antallAnsatte] = nyAnsatt;
    antallAnsatte++;
    if (nyAnsatt.hentKjnn() == Ansatt.KVINNE)
      antallKvinner++;
  }

  // Selektorer
  int hentAntallAnsatte()     { return antallAnsatte; }
  int hentAntallKvinner()     { return antallKvinner; }
  Ansatt[] hentAnsattTabell() { return ansattTabell; }

  // Returnerer ansatt som har angitt nummer.
  Ansatt hentAnsatt(int nummer) {
    if (!lovligIndeks(nummer)) {
      System.out.println("Ulovlig ansatt-nummer: " + nummer);
      return null;
    }
    return ansattTabell[nummer];
  }

  // En ansatt med angitt nummer erstattes med en ny ansatt.
  void settAnsatt(int nummer, Ansatt nyAnsatt) {
    if (!lovligIndeks(nummer)) {
      System.out.println("Ulovlig ansatt-nummer: " + nummer);
      return;
    }
    ansattTabell[nummer] = nyAnsatt;
  }

  // Beregner kvinneandel i firma.
  double hentProsentandelKvinner() {
    if (ansattTabellTom()) {
      System.out.println("Ingen ansatt registrert i personellregisteret");
      return 0.0;
    }
    return 100.0 * antallKvinner / antallAnsatte;
  }

  // Returnerer statistikk over firma.
  @Override
  public String toString() {
    return String.format("Firmaet har %d ansatte, hvorav %.2f%% er kvinner.",
        hentAntallAnsatte(), hentProsentandelKvinner());
  }
}